import { v4 as uuidv4 } from 'uuid';
import fs from 'fs';
import path from 'node:path';
import fetch from 'node-fetch';
import express from 'express';
import * as Misskey from 'misskey-js';

let app = express();
let port = process.env.PORT || 4000;

const config = JSON.parse(fs.readFileSync(path.resolve("config", "config.json")));
let host = config.host;

let app_name = config.app.name;
let app_callback = config.app.callback;
let app_permission = config.app.permission;

// https://misskey.io/miauth/c1f6d42b-468b-4fd2-8274-e58abdedef6f?name=MyApp&callback=https%3A%2F%2Fmyapp.example.com%2Fcallback&permission=write:notes,write:following,read:drive

let session_id = uuidv4();
let auth_url = `${host}/miauth/${session_id}?name=${app_name}&callback=${app_callback}&permission=${app_permission}`;
let auth_status = 0;

console.log(auth_url);

app.get('/callback', function(req, res) {
    res.send({
        "status": 200,
        "success": true,
    })

    auth_status = 1;
});

async function check_auth_status() {
    return await new Promise(resolve => {
        const check = setInterval(() => {
            console.log("Checking auth status...");
    
            if(auth_status == 1) {
                console.log("Bot has been authenticated");
                resolve(true);
                clearInterval(check);
                return true;
            }
        }, 2000);
    });
};

async function get_access_token(host, session_id) {
    const response = await fetch(`${host}/api/miauth/${session_id}/check`, {
        method: 'POST'
    });
    
    const data = await response.json();

    const res_status = data.ok;
    const res_token = data.token;
    

    if(res_status) {
        return res_token;
    } else {
        return false; // error occured
    }
};

(async () => {
    await check_auth_status()
    let access_token = await get_access_token(host, session_id);

    console.log(access_token);
})();

app.listen(port);
console.log(`Callback URL: http://localhost:${port}/callback`);